function polyphs2
% function demonstrates 1-to-5 upsampling polyphase fir filter, 
% shows phase and amplitude responses of polyphase partition. Tracks a sinusoid
% at various points in 1-to-5 up sample process.
% Script file written by fred harris of SDSU. Copyright 2002

clf
nfrq=200;
aa=[1,zeros(1,139)];

[bb,aa]=remez(139,2*[0 .088 .112 .50],[1 1 0 0]);
[h,w]=freqz(bb,aa,500);

zplane(bb,aa);
grid
title(['Pole-Zero Plot: 140-Pnt FIR Filter'])
pause

[h,w]=freqz(bb,aa,nfrq);
subplot(1,2,1);
plot(w/(2*pi),abs(h)/max(abs(h)));
axis([0 .5-1/nfrq 0 1.1]);
xlabel('Normalized Frequency');
ylabel('Magnitude');
title(['Frequency Response: 140-Pnt FIR']);
grid;
subplot(1,2,2);
plot(w/(2*pi),20*log10(abs(h/max(h))));
axis([0 .5-1/nfrq -80 5]);
xlabel('Normalized Frequency');
ylabel('Log Magnitude (dB)');
title(['Frequency Response: 140-Pnt FIR']);
grid;

pause
%x=zeros(1,nn);
%x=[1 x];
%y=filter(bb,aa,x);
y=bb;
ym=max(y);
subplot(1,2,1);
plot(y/ym);
v1=[0 140 -.4 1.1];
axis(v1);
xlabel('normalized time');
ylabel('Amplitude');
title(['Impulse Response: 140-Pnt FIR']);
grid;
subplot(1,2,2);
plot(20*log10(abs(y/ym)));
v2=[0 140 -60 0];
axis(v2);
xlabel('Normalized Time');
ylabel('Log Magnitude (dB)');
title(['Impulse Response: 140-Pnt FIR']);
grid;

hh=reshape(bb,5,28);

h1=hh(1,:);subplot(3,2,1);plot(h1/ym);xlabel('time');ylabel('ampl');title('phase 1');axis([0 30 -.3 +1.2]);grid
h2=hh(2,:);subplot(3,2,2);plot(h2/ym);xlabel('time');ylabel('ampl');title('phase 2');axis([0 30 -.3 +1.2]);grid
h3=hh(3,:);subplot(3,2,3);plot(h3/ym);xlabel('time');ylabel('ampl');title('phase 3');axis([0 30 -.3 +1.2]);grid
h4=hh(4,:);subplot(3,2,4);plot(h4/ym);xlabel('time');ylabel('ampl');title('phase 4');axis([0 30 -.3 +1.2]);grid
h5=hh(5,:);subplot(3,2,5);plot(h5/ym);xlabel('time');ylabel('ampl');title('phase 5');axis([0 30 -.3 +1.2]);grid
subplot(3,2,6);plot(bb/ym);           xlabel('time');ylabel('ampl');title('prototype');axis([0 140 -.3 +1.2]);grid
pause

clf
fh1=20*log10(abs(fft(hh(1,:),128)));subplot(3,2,1);plot([0:1/128:1-1/128],fh1);
xlabel('freq');ylabel('log mag (dB)');title('phase 1');grid
fh2=20*log10(abs(fft(hh(2,:),128)));subplot(3,2,2);plot([0:1/128:1-1/128],fh2);
xlabel('freq');ylabel('log mag (dB)');title('phase 2');grid
fh3=20*log10(abs(fft(hh(3,:),128)));subplot(3,2,3);plot([0:1/128:1-1/128],fh3);
xlabel('freq');ylabel('log mag (dB)');title('phase 3');grid
fh4=20*log10(abs(fft(hh(4,:),128)));subplot(3,2,4);plot([0:1/128:1-1/128],fh4);
xlabel('freq');ylabel('log mag (dB)');title('phase 4');grid
fh5=20*log10(abs(fft(hh(5,:),128)));subplot(3,2,5);plot([0:1/128:1-1/128],fh5);
xlabel('freq');ylabel('log mag (dB)');title('phase 5');grid
fbb=20*log10(abs(fft(bb,512)));     subplot(3,2,6);plot([0:1/512:1-1/512],fbb);vv=[0 1 -80 5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('prototype');grid
pause;



subplot(3,2,1);plot([0:1/128:1-1/128],fh1);vv=[0 1 -14.5 -13.5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('phase 1');grid
subplot(3,2,2);plot([0:1/128:1-1/128],fh2);vv=[0 1 -14.5 -13.5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('phase 2');grid
subplot(3,2,3);plot([0:1/128:1-1/128],fh3);vv=[0 1 -14.5 -13.5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('phase 3');grid
subplot(3,2,4);plot([0:1/128:1-1/128],fh4);vv=[0 1 -14.5 -13.5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('phase 4');grid
subplot(3,2,5);plot([0:1/128:1-1/128],fh5);vv=[0 1 -14.5 -13.5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('phase 5');grid
subplot(3,2,6);plot([0:1/512:1-1/512],fbb);vv=[0 1 -.5 .5];axis(vv);
xlabel('freq');ylabel('log mag (dB)');title('prototype');grid

pause;
clf


fh1=unwrap(angle(fft([hh(1,:)],256)))*180/pi;subplot(3,2,1);plot([0:1/256:1-1/256],fh1);
xlabel('freq');ylabel('phase (deg)');title('phase 1');grid
fh2=unwrap(angle(fft([0 hh(2,:)],256)))*180/pi;subplot(3,2,2);plot([0:1/256:1-1/256],fh2);
xlabel('freq');ylabel('phase (deg)');title('phase 2');grid
fh3=unwrap(angle(fft([0 0 hh(3,:)],256)))*180/pi;subplot(3,2,3);plot([0:1/256:1-1/256],fh3);
xlabel('freq');ylabel('phase (deg)');title('phase 3');grid
fh4=unwrap(angle(fft([0 0 0 hh(4,:)],256)))*180/pi;subplot(3,2,4);plot([0:1/256:1-1/256],fh4);
xlabel('freq');ylabel('phase (deg)');title('phase 4');grid
fh5=unwrap(angle(fft([0 0 0 0 hh(5,:)],256)))*180/pi;subplot(3,2,5);plot([0:1/256:1-1/256],fh5);
xlabel('freq');ylabel('phase (deg)');title('phase 5');grid
fbb=unwrap(angle(fft(bb,512)))*180/pi;     subplot(3,2,6);plot([0:1/512:1-1/512],fbb);%vv=[0 1 -80 5];axis(vv);
xlabel('freq');ylabel('phase (deg)');title('prototype');grid
pause;

clf
plot([0:1/256:.5],fh1(1:129));hold
plot([0:1/256:.5],fh2(1:129));
plot([0:1/256:.5],fh3(1:129));
plot([0:1/256:.5],fh4(1:129));
plot([0:1/256:.5],fh5(1:129));
xlabel('freq');ylabel('phase (deg)');title('phase of 5-polyphase segments');
hold;grid;pause;

xin=sin(2*pi*(0:99)/7.3);
subplot(2,1,1);plot(xin(1:100));xlabel('time');ylabel('amplitude');title('input x(n)');
axis([0 100 -1.2 1.2]);
grid

w1=kaiser(100,6);

fxin=20*log10(abs(fft(w1'.*xin,200)/100));
subplot(2,1,2);plot([0:1/200:1-1/200],fxin);axis([0 1 -80 5]);
xlabel('frequency');ylabel('log magnitude (dB)');title('input spectrum X(f)');
grid
pause

clg
ns=size(h1);aa=[1, zeros(1,ns-1)];
y1=filter(h1,aa,xin);subplot(3,2,1);plot(y1(1:100));vv=[0 100 -.4 .4];axis(vv);
xlabel('time');ylabel('amplitude');title('output y1(n)');grid
y2=filter(h2,aa,xin);subplot(3,2,2);plot(y2(1:100));vv=[0 100 -.4 .4];axis(vv);
xlabel('time');ylabel('amplitude');title('output y2(n)');grid
y3=filter(h3,aa,xin);subplot(3,2,3);plot(y3(1:100));vv=[0 100 -.4 .4];axis(vv);
xlabel('time');ylabel('amplitude');title('output y3(n)');grid
y4=filter(h4,aa,xin);subplot(3,2,4);plot(y4(1:100));vv=[0 100 -.4 .4];axis(vv);
xlabel('time');ylabel('amplitude');title('output y4(n)');grid
y5=filter(h5,aa,xin);subplot(3,2,5);plot(y5(1:100));vv=[0 100 -.4 .4];axis(vv);
xlabel('time');ylabel('amplitude');title('output y5(n)');grid
yy=filter(bb,aa,xin); subplot(3,2,6);plot(yy(1:100));vv=[0 100 -1.2 1.2];axis(vv);grid
xlabel('time');ylabel('amplitude');title('output y(n) no upsampling');
ymx=[y1;y2;y3;y4;y5];
pause

ymxx=reshape(ymx,1,500);plot(ymxx(1:200));vv=[0 200 -.4 .4];axis(vv);grid
xlabel('time');ylabel('amplitude');title('output y(n) with 1-to-5 upsampling');

pause
clf

subplot(2,1,1);plot(5*ymxx(1:200));v1=[0 200 -1.2 +1.2]; axis(v1);grid
xlabel('time');ylabel('amplitude');title('output y(n) with 1-to-5 upsampling');
w1=kaiser(500,6);

fxin=20*log10(abs(fft(w1'.*ymxx,1000)/100));
subplot(2,1,2);plot([0:1/1000:1-1/1000],fxin);vv=[0 1 -80 5];axis(vv);grid
xlabel('frequency');ylabel('log magnitude (dB)');title('spectrum output y(n) with 1-to-5 upsampling');
pause

clf
w1=kaiser(500,6);
zz=zeros(1,100);
xinm=[xin;zz;zz;zz;zz];
xinmm=reshape(xinm,1,500);

subplot(2,1,1);plot(xinmm(1:100));xlabel('time');ylabel('amplitude');title('input x(n) with 1-to-5 upsampling');
v1=[0 100 -1.2 1.2];axis(v1);grid
fxinmm=20*log10(abs(fft(w1'.*xinmm,1000)/500));
subplot(2,1,2);plot([0:1/1000:1-1/1000],fxinmm);vv=[0 1 -80 5];axis(vv);grid
xlabel('frequency');ylabel('log magnitude (dB)');title('input spectrum X(f) after 1-to-5 upsampling');

